import { RcSdk } from "./RcSdk";

interface ITeamRemoveMemberBody {
    teamId: string;
    userId: string;
}

interface ITeamAddMemberBody {
    teamId: string;
    members: Array<{
        userId: string;
        roles: Array<string>;
    }>;
}

interface ITeamDeleteBody {
    teamId: string;
    roomsToRemove?: Array<string>;
}

interface ITeamCreateBody {
    name: string;
    type: 0 | 1;
    members?: Array<string>;
    room?: {
        readOnly: boolean;
    };
}

export interface ITeam {
    _id: string;
    name: string;
    active?: boolean;
    createdAt: string;
    _updatedAt?: string;
}

export interface ITeamMember {
    user: {
        _id: string;
        name: string;
        username: string;
    };
    createdAt: string;
}

interface ITeamListAllResponse {
    teams: Array<ITeam>;
    count: number;
    offset: number;
    total: number;
    success: boolean;
    error?: string;
}

interface ITeamMembersResponse {
    members: Array<ITeamMember>;
    count: number;
    offset: number;
    total: number;
    success: boolean;
    error?: string;
}

interface ITeamInfoResponse {
    teamInfo: ITeam;
    success: boolean;
    error?: string;
}

interface ITeamDeleteResponse {
    teamInfo: ITeam;
    success: boolean;
    error?: string;
}

interface ITeamCreateResponse {
    team: ITeam;
    success: boolean;
    error?: string;
}

interface ITeamMemberResponse {
    success: boolean;
    error?: string;
}

export class RcSdkTeam {
    private sdk: RcSdk;
    constructor(sdk: RcSdk) {
        this.sdk = sdk;
    }

    public async listAll(): Promise<ITeamListAllResponse> {
        const response = await this.sdk.get(`teams.listAll`);
        return this.sdk.parseResponse(response);
    }

    public async members(teamId: string): Promise<ITeamMembersResponse> {
        const response = await this.sdk.get(`teams.members?teamId=${teamId}`);
        return this.sdk.parseResponse(response);
    }

    public async info(teamId: string): Promise<ITeamInfoResponse> {
        const response = await this.sdk.get(`teams.info?teamId=${teamId}`);
        return this.sdk.parseResponse(response);
    }

    public async delete(body: ITeamDeleteBody): Promise<ITeamDeleteResponse> {
        const response = await this.sdk.post(`teams.delete`, body);
        return this.sdk.parseResponse(response);
    }

    public async create(body: ITeamCreateBody): Promise<ITeamCreateResponse> {
        const response = await this.sdk.post(`teams.create`, body);
        return this.sdk.parseResponse(response);
    }

    public async removeMember(
        body: ITeamRemoveMemberBody,
    ): Promise<ITeamMemberResponse> {
        const response = await this.sdk.post(`teams.removeMember`, body);
        return this.sdk.parseResponse(response);
    }

    public async addMembers(
        body: ITeamAddMemberBody,
    ): Promise<ITeamMemberResponse> {
        const response = await this.sdk.post(`teams.addMembers`, body);
        return this.sdk.parseResponse(response);
    }
}
